from . import infamous_lab
from . import cie_standard
from . import cie_monitor_helpers
from . import fairchildL
from . import funny_dude_discovers_maxRGB_in_adobe
from . import G0_brilliance
from . import ware_cowan
from . import lms
from . import maxRGB_revolution
from . import sumrgb
from . import nayatani_insanity
from . import radiance
from . import uv_space
from . import cmfs
from . import plotting
from . import brightness_ranking
from . import lch
from . import guth_vector
from . import jazzy
from . import hunt
from . import spectra
from . import illuminants
from . import hsv
from . import jingHSP
from . import svf
from . import hue_angles
from . import matlab
from . import img_stats
from . import atd95
from . import strobe_export
from . import monitor_fitting
from . import alices_damn_monitor

# from . import nayatani_test
# from . import hunt_test
# from . import atd95_test

cie_standard.generatePlankLasso()